package com.ca.chorus.db;

import java.sql.ResultSet;
import java.sql.Statement;
import java.util.List;
import java.util.concurrent.Semaphore;

import org.junit.Before;
import org.junit.Test;
import org.teiid.adminapi.impl.ModelMetaData;
import org.teiid.jdbc.ConnectionImpl;
import org.teiid.jdbc.PreparedStatementImpl;
import org.teiid.jdbc.RequestOptions;
import org.teiid.jdbc.StatementCallback;
import org.teiid.language.Call;
import org.teiid.metadata.MetadataFactory;
import org.teiid.metadata.Procedure;
import org.teiid.metadata.RuntimeMetadata;
import org.teiid.runtime.EmbeddedConfiguration;
import org.teiid.runtime.EmbeddedServer;
import org.teiid.translator.DataNotAvailableException;
import org.teiid.translator.ExecutionContext;
import org.teiid.translator.ExecutionFactory;
import org.teiid.translator.ProcedureExecution;
import org.teiid.translator.Translator;
import org.teiid.translator.TranslatorException;
import org.teiid.translator.TypeFacility;

public class TestContinuousExecutionWithEmptyResultSet
{
    static public class EmptyResultSetStoredProc
        implements
            ProcedureExecution
    {

        @Override
        public List<?> next() throws TranslatorException, DataNotAvailableException
        {
            System.out.println("next");
            return null;
        }

        @Override
        public void close()
        {
            System.out.println("close");
        }

        @Override
        public void cancel() throws TranslatorException
        {
            //
        }

        @Override
        public void execute() throws TranslatorException
        {
            System.out.println("execute");
        }

        @Override
        public List<?> getOutputParameterValues() throws TranslatorException
        {
            return null;
        }

    }

    @Translator(name = "testing")
    static public class EmptyResultSetTranslator
        extends ExecutionFactory<Void, Void>
    {
        public EmptyResultSetTranslator()
        {
            setSourceRequired(false);
        }

        @Override
        public void getMetadata(final MetadataFactory metadataFactory, final Void conn) throws TranslatorException
        {
            final Procedure emptyResultSetStoredProc = metadataFactory.addProcedure("empty_result_set");
            metadataFactory.addProcedureResultSetColumn("dummy",
                                                        TypeFacility.getDataTypeName(Integer.class),
                                                        emptyResultSetStoredProc);
        }

        @Override
        public ProcedureExecution createProcedureExecution(final Call command,
                                                           final ExecutionContext executionContext,
                                                           final RuntimeMetadata metadata,
                                                           final Void connection) throws TranslatorException
        {
            System.out.println("createProcedureExecution");
            return new EmptyResultSetStoredProc();
        }

    }

    private EmbeddedServer server;

    @Before
    public void setup() throws Exception
    {
        this.server = new EmbeddedServer();
        final ModelMetaData testingModelMetaData = new ModelMetaData();
        testingModelMetaData.setName("testing");
        testingModelMetaData.setSchemaSourceType("native");
        testingModelMetaData.addSourceMapping("testing", "testing", null);
        final ModelMetaData dataWindowModelMetaData = new ModelMetaData();
        dataWindowModelMetaData.setName("dw");
        dataWindowModelMetaData.setSchemaSourceType("native");
        dataWindowModelMetaData.addSourceMapping("dw", "data-pipe", "dw");

        this.server.addTranslator(new EmptyResultSetTranslator());
        this.server.start(new EmbeddedConfiguration());
        this.server.deployVDB("test", testingModelMetaData);
    }

    @Test(timeout = 5000)
    public void emptyResultSet() throws Exception
    {
        final ConnectionImpl connection = this.server.getDriver().connect("jdbc:teiid:test", null);
        final PreparedStatementImpl call = connection.prepareStatement("CALL empty_result_set()");
        final Semaphore semaphore = new Semaphore(-1);
        call.submitExecute(new StatementCallback()
        {

            @Override
            public void onRow(final Statement s, final ResultSet rs) throws Exception
            {
                semaphore.drainPermits();
                System.out.println("onRow");
            }

            @Override
            public void onException(final Statement s, final Exception e) throws Exception
            {
                semaphore.drainPermits();
                System.out.println("onException");
            }

            @Override
            public void onComplete(final Statement s) throws Exception
            {
                semaphore.drainPermits();
                System.out.println("onComplete");
            }
        }, new RequestOptions().continuous(true));
        semaphore.acquire();
        System.out.println("leaving test");
    }
}
